@@ -45,7 +45,16 @@
 
 #ifdef ARCH_X86
 
+#ifndef __x86_64__
 #define rdtscll(val) __asm__ __volatile__("rdtsc" : "=A" (val))
+#else
+#define rdtscll(val) \
+    do { \
+        uint32_t tickl, tickh; \
+        __asm__ __volatile__("rdtsc" : "=a" (tickl), "=d" (tickh)); \
+        val = ((uint64_t)tickh << 32) | tickl; \
+    } while (0)
+#endif
 
 double cpuinfo_get_speed( void )
 {
@@ -81,11 +90,19 @@ static cpuid_regs_t cpuid( int func ) {
             : "g" (func)
             : "%eax", "%ebx", "%ecx", "%edx");
 #else
+#ifndef __x86_64__
     asm("movl %%ebx, %%esi; movl %4,%%eax; " CPUID
         "movl %%eax,%0; movl %%ebx,%1; movl %%ecx,%2; movl %%edx,%3; movl %%esi, %%ebx"
             : "=m" (regs.eax), "=m" (regs.ebx), "=m" (regs.ecx), "=m" (regs.edx)
             : "g" (func)
             : "%eax", "%ecx", "%edx", "%esi");
+#else
+    asm("movq %%rbx, %%rsi; movl %4,%%eax; " CPUID
+        "movl %%eax,%0; movl %%ebx,%1; movl %%ecx,%2; movl %%edx,%3; movq %%rsi, %%rbx"
+            : "=m" (regs.eax), "=m" (regs.ebx), "=m" (regs.ecx), "=m" (regs.edx)
+            : "g" (func)
+            : "%eax", "%ecx", "%edx", "%esi");
+#endif
 #endif
     return regs;
 }
@@ -196,9 +213,7 @@ static void store32( char *d, unsigned int v )
 void cpuinfo_print_info( void )
 {
     cpuid_regs_t regs, regs_ext;
-    unsigned int max_cpuid;
     unsigned int max_ext_cpuid;
-    unsigned int amd_flags;
     int family, model, stepping;
     char idstr[13];
     char *model_name;
@@ -206,7 +221,6 @@ void cpuinfo_print_info( void )
     int i;
 
     regs = cpuid(0);
-    max_cpuid = regs.eax;
 
     store32(idstr+0, regs.ebx);
     store32(idstr+4, regs.edx);
@@ -225,7 +239,6 @@ void cpuinfo_print_info( void )
 
     if (max_ext_cpuid >= (1<<31) + 1) {
         regs_ext = cpuid((1<<31) + 1);
-        amd_flags = regs_ext.edx;
 
         if (max_ext_cpuid >= (1<<31) + 4) {
             for (i = 2; i <= 4; i++) {
@@ -238,8 +251,6 @@ void cpuinfo_print_info( void )
             processor_name[48] = 0;
             model_name = processor_name;
         }
-    } else {
-        amd_flags = 0;
     }
 
     /* Is this dangerous? */
