@@ -191,11 +191,9 @@ static void osd_string_render_bordered_image4444( osd_string_t *osds, const char
 {
     int stringwidth, stringheight;
     ft_font_t *font = osd_font_get_font( osds->font );
-    int left_x, right_x;
-    int top_y, bottom_y;
+    int right_x, bottom_y;
     int textpos_x, textpos_y;
 
-    left_x = top_y = 0;
     right_x = ft_font_points_to_subpix_width( font, 2 );
     bottom_y = 2;
     textpos_x = ft_font_points_to_subpix_width( font, 1 );
@@ -380,13 +378,12 @@ static int load_png_to_packed4444( uint8_t *buffer, int *active,
                                    double pixel_aspect, pnginput_t *pngin )
 {
     int has_alpha = pnginput_has_alpha( pngin );
-    int pngwidth, pngheight;
+    int pngwidth;
     uint8_t *cb444;
     uint8_t *curout;
     int i;
 
     pngwidth = pnginput_get_width( pngin );
-    pngheight = pnginput_get_height( pngin );
 
     cb444 = malloc( pngwidth * 3 );
     if( !cb444 ) return 0;
