@@ -257,7 +257,6 @@ int determine_pulldown_offset_history_new( int top_repeat, int bot_repeat, int t
     int minbotpos = -1;
     int min2botval = -1;
     int min2botpos = -1;
-    int predicted_pos = 0;
 
     tophistory[ histpos ] = top_repeat;
     bothistory[ histpos ] = bot_repeat;
@@ -269,12 +268,6 @@ int determine_pulldown_offset_history_new( int top_repeat, int bot_repeat, int t
     avgtop /= 5;
     avgbot /= 5;
 
-    for( i = 0; i < 5; i++ ) { if( (1<<i) == predicted ) { predicted_pos = i; break; } }
-
-    /*
-    fprintf( stderr, "top: %8d bot: %8d\ttop-avg: %8d bot-avg: %8d (%d)\n", top_repeat, bot_repeat, top_repeat - avgtop, bot_repeat - avgbot, (5 + predicted_pos - reference) % 5 );
-    */
-
     for( j = 0; j < HISTORY_SIZE; j++ ) {
         int cur = tophistory[ j ];
         if( cur < mintopval || mintopval < 0 ) {
@@ -365,7 +358,6 @@ int determine_pulldown_offset_short_history_new( int top_repeat, int bot_repeat,
     int minbotpos = -1;
     int min2botval = -1;
     int min2botpos = -1;
-    int predicted_pos = 0;
 
     tophistory[ histpos ] = top_repeat;
     bothistory[ histpos ] = bot_repeat;
@@ -377,14 +369,6 @@ int determine_pulldown_offset_short_history_new( int top_repeat, int bot_repeat,
     avgtop /= 3;
     avgbot /= 3;
 
-    for( i = 0; i < 5; i++ ) { if( (1<<i) == predicted ) { predicted_pos = i; break; } }
-
-    /*
-    fprintf( stderr, "top: %8d bot: %8d\ttop-avg: %8d bot-avg: %8d (%d)\n",
-             top_repeat, bot_repeat, top_repeat - avgtop, bot_repeat - avgbot,
-             (5 + predicted_pos - reference) % 5 );
-    */
-
     for( j = 0; j < 3; j++ ) {
         int cur = tophistory[ (histpos + 5 - j) % 5 ];
         if( cur < mintopval || mintopval < 0 ) {
