@@ -47,7 +47,6 @@
 #endif
 
 #include "speedy.h"
-#include "speedtools.h"
 #include "copyfunctions.h"
 #include "attributes.h"
 #include "mmx.h"
@@ -1036,9 +1035,6 @@ static void composite_packed4444_alpha_to_packed422_scanline_mmxext( uint8_t *ou
         return;
     }
 
-    READ_PREFETCH_2048( input );
-    READ_PREFETCH_2048( foreground );
-
     movq_m2r( alpha, mm2 );
     pshufw_r2r( mm2, mm2, 0 );
     pxor_r2r( mm7, mm7 );
@@ -1166,9 +1162,6 @@ static void composite_packed4444_to_packed422_scanline_mmxext( uint8_t *output,
     const mmx_t round  = { 0x0080008000800080ULL };
     int i;
 
-    READ_PREFETCH_2048( input );
-    READ_PREFETCH_2048( foreground );
-
     pxor_r2r( mm7, mm7 );
     for( i = width/2; i; i-- ) {
         int fg1 = *((uint32_t *) foreground);
