@@ -51,6 +51,14 @@
 #define MAX_KEYSYMS 350
 #define MAX_BUTTONS 15
 
+#ifdef __linux__ /* This depends on sysfs, so it is linux only */
+#define DEFAULT_VIDEO_DEV "auto"
+#define DEFAULT_ALSA_INPUT "auto"
+#else
+#define DEFAULT_VIDEO_DEV "/dev/video0"
+#define DEFAULT_ALSA_INPUT "hw:1,0"
+#endif
+
 struct config_s
 {
     char *geometry;
@@ -96,7 +104,6 @@ struct config_s
     char *freq;
     char *ssdir;
     char *timeformat;
-    char *audiomode;
     char *xmltvfile;
     char *xmltvlanguage;
     unsigned int channel_text_rgb;
@@ -118,12 +125,14 @@ struct config_s
     int slave_mode;
 
     double overscan;
+    char *matte;
 
     char *config_filename;
     xmlDocPtr doc;
 
     char *alsa_inputdev;
     char *alsa_outputdev;
+    int alsa_latency;
 };
 
 static unsigned int parse_colour( const char *str )
@@ -261,11 +270,6 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             ct->send_fields = atoi( curval );
         }
 
-        if( !xmlStrcasecmp( name, BAD_CAST "AudioMode" ) ) {
-            if( ct->audiomode ) free( ct->audiomode );
-            ct->audiomode = strdup( curval );
-        }
-
         if( !xmlStrcasecmp( name, BAD_CAST "XMLTVFile" ) ) {
             if( ct->xmltvfile ) free( ct->xmltvfile );
             ct->xmltvfile = expand_user_path( curval );
@@ -406,6 +410,11 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             setlocale( LC_NUMERIC, prevloc );
         }
 
+	if( !xmlStrcasecmp( name, BAD_CAST "Matte" ) ) {
+	    if( ct->matte ) free( ct->matte );
+	    ct->matte = strdup( curval );
+	}
+
         if( !xmlStrcasecmp( name, BAD_CAST "MixerDevice" ) ) {
             if( ct->mixerdev ) free( ct->mixerdev );
             ct->mixerdev = strdup( curval );
@@ -421,6 +430,9 @@ static void parse_option( config_t *ct, xmlNodePtr node )
             ct->alsa_outputdev = strdup( curval );
         }
 
+	if( !xmlStrcasecmp( name, BAD_CAST "AlsaLatency" ) ) {
+	    ct->alsa_latency = atoi( curval );
+	}
     }
 
     if( name ) xmlFree( name );
@@ -612,7 +624,7 @@ static void print_usage( char **argv )
     lfputs( _("  -A, --nowidescreen         4:3 mode.\n"), stderr );
     lfputs( _("  -b, --vbidevice=DEVICE     VBI device (defaults to /dev/vbi0).\n"), stderr );
     lfputs( _("  -c, --channel=CHANNEL      Tune to the specified channel on startup.\n"), stderr );
-    lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to /dev/video0).\n"), stderr );
+    lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to " DEFAULT_VIDEO_DEV ").\n"), stderr );
     lfputs( _("  -f, --frequencies=NAME     The frequency table to use for the tuner.\n"
               "                             (defaults to us-cable).\n\n"
               "                             Valid values are:\n"
@@ -667,7 +679,7 @@ static void print_config_usage( char **argv )
     lfputs( _("  -A, --nowidescreen         4:3 mode.\n"), stderr );
     lfputs( _("  -b, --vbidevice=DEVICE     VBI device (defaults to /dev/vbi0).\n"), stderr );
     lfputs( _("  -c, --channel=CHANNEL      Tune to the specified channel on startup.\n"), stderr );
-    lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to /dev/video0).\n"), stderr );
+    lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to " DEFAULT_VIDEO_DEV ").\n"), stderr );
     lfputs( _("  -f, --frequencies=NAME     The frequency table to use for the tuner.\n"
               "                             (defaults to us-cable).\n\n"
               "                             Valid values are:\n"
@@ -711,18 +723,19 @@ static void print_config_usage( char **argv )
               "                                 video, radio, monitor\n"), stderr );
     lfputs( _("  -p, --alsainputdev=DEV     Specifies an ALSA device to read input on\n"
               "                                 Examples:\n"
-              "                                          hw:1,0\n"
+	      "                                          " DEFAULT_ALSA_INPUT "\n"
               "                                          disabled\n"), stderr );
     lfputs( _("  -P, --alsaoutputdev=DEV    Specifies an ALSA device to write output to\n"
               "                                 Examples:\n"
               "                                          hw:0,0\n"
               "                                          disabled\n"), stderr );
+    lfputs( _("  -z, --alsalatency=LATENCY  Specifies ALSA loopback latency in milli-seconds\n"), stderr );
 }
 
 static void print_scanner_usage( char **argv )
 {
     lfprintf( stderr, _("usage: %s [OPTION]...\n\n"), argv[ 0 ]);
-    lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to /dev/video0).\n"), stderr );
+    lfputs( _("  -d, --device=DEVICE        video4linux device (defaults to " DEFAULT_VIDEO_DEV ").\n"), stderr );
     lfputs( _("  -F, --configfile=FILE      Additional config file to load settings from.\n"), stderr );
     lfputs( _("  -h, --help                 Show this help message.\n"), stderr );
     lfputs( _("  -i, --input=INPUTNUM       video4linux input number (defaults to 0).\n"), stderr );
@@ -780,7 +793,7 @@ config_t *config_new( void )
 
     ct->inputwidth = 720;
     ct->inputnum = 0;
-    ct->v4ldev = strdup( "/dev/video0" );
+    ct->v4ldev = strdup( DEFAULT_VIDEO_DEV );
     ct->norm = strdup( "ntsc" );
     ct->freq = strdup( "us-cable" );
     temp_dirname = getenv( "HOME" );
@@ -789,7 +802,6 @@ config_t *config_new( void )
     } else {
         ct->ssdir = 0;
     }
-    ct->audiomode = strdup( "stereo" );
     ct->xmltvfile = strdup( "none" );
     ct->xmltvlanguage = strdup( "none" );
     ct->timeformat = strdup( "%X" );
@@ -812,12 +824,14 @@ config_t *config_new( void )
     ct->slave_mode = 0;
 
     ct->overscan = 0.0;
+    ct->matte = NULL;
 
     ct->config_filename = 0;
     ct->doc = 0;
 
-    ct->alsa_inputdev = strdup( "hw:1,0" );
-    ct->alsa_outputdev = strdup( "hw:0,0" );
+    ct->alsa_inputdev = strdup( DEFAULT_ALSA_INPUT );
+    ct->alsa_outputdev = strdup( "default" );
+    ct->alsa_latency = 50;
 
     /* Default key bindings. */
     ct->keymap[ 0 ] = TVTIME_NOCOMMAND;
@@ -843,6 +857,7 @@ config_t *config_new( void )
     ct->keymap[ 'd' ] = TVTIME_SHOW_STATS;
     ct->keymap[ 'g' ] = TVTIME_SHOW_EPG;
     ct->keymap[ 'a' ] = TVTIME_TOGGLE_ASPECT;
+    ct->keymap[ I_INSERT ] = TVTIME_TOGGLE_MATTE;
     ct->keymap[ 'f' ] = TVTIME_TOGGLE_FULLSCREEN;
     ct->keymap[ 'i' ] = TVTIME_TOGGLE_INPUT;
     ct->keymap[ 's' ] = TVTIME_SCREENSHOT;
@@ -1092,6 +1107,7 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
         { "priority", 2, 0, 'R' },
         { "alsainputdev", 2, 0, 'p' },
         { "alsaoutputdev", 2, 0, 'P' },
+	{ "alsalatency", 2, 0, 'z' },
         { 0, 0, 0, 0 }
     };
     int option_index = 0;
@@ -1104,7 +1120,7 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
     }
 
     while( (c = getopt_long( argc, argv,
-            "aAhmMF:g:I:d:b:i:c:n:D:f:x:t:Ll:R:p:P",
+	    "aAhmMF:g:I:d::b::i::c:n::D:f::x:t::Ll::R::p::P::z::",
             long_options, &option_index )) != -1 ) {
         switch( c ) {
         case 'a': ct->aspect = 1; break;
@@ -1206,6 +1222,13 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
                       ct->alsa_outputdev = strdup( optarg );
                   }
                   break;
+	case 'z': if( !optarg ) {
+		      fprintf( stdout, "AlsaLatency:%d\n",
+			       config_get_alsa_latency( ct ) );
+		  } else {
+		      ct->alsa_latency = atoi( optarg );
+		  }
+		  break;
         default:
             print_config_usage( argv );
             return 0;
@@ -1277,6 +1300,8 @@ int config_parse_tvtime_config_command_line( config_t *ct, int argc, char **argv
 
         config_save( ct, "AlsaInputDev", ct->alsa_inputdev );
         config_save( ct, "AlsaOutputDev", ct->alsa_outputdev );
+	snprintf( tempstring, sizeof( tempstring ), "%d", ct->alsa_latency );
+	config_save( ct, "AlsaLatency", tempstring );
     }
 
     return 1;
@@ -1331,12 +1356,12 @@ void config_free_data( config_t *ct )
     if( ct->norm ) free( ct->norm );
     if( ct->freq ) free( ct->freq );
     if( ct->ssdir ) free( ct->ssdir );
-    if( ct->audiomode ) free( ct->audiomode );
     if( ct->xmltvfile ) free( ct->xmltvfile );
     if( ct->xmltvlanguage ) free( ct->xmltvlanguage );
     if( ct->timeformat ) free( ct->timeformat );
     if( ct->mixerdev ) free( ct->mixerdev );
     if( ct->vbidev ) free( ct->vbidev );
+    if( ct->matte ) free( ct->matte );
     if( ct->config_filename ) free( ct->config_filename );
     if( ct->deinterlace_method ) free( ct->deinterlace_method );
     if( ct->alsa_inputdev ) free( ct->alsa_inputdev );
@@ -1626,6 +1651,11 @@ double config_get_overscan( config_t *ct )
     return ct->overscan;
 }
 
+const char *config_get_matte( config_t *ct )
+{
+    return ct->matte;
+}
+
 int config_get_ntsc_cable_mode( config_t *ct )
 {
     return ct->ntsc_mode;
@@ -1681,11 +1711,6 @@ int config_get_global_hue( config_t *ct )
     return ct->hue;
 }
 
-const char *config_get_audio_mode( config_t *ct )
-{
-    return ct->audiomode;
-}
-
 const char *config_get_xmltv_file( config_t *ct )
 {
     return ct->xmltvfile;
@@ -1754,3 +1779,8 @@ const char *config_get_alsa_outputdev( config_t *ct )
 {
     return ct->alsa_outputdev;
 }
+
+int config_get_alsa_latency( config_t *ct )
+{
+    return ct->alsa_latency;
+}
