@@ -177,10 +177,10 @@ int config_get_global_brightness( config_t *ct );
 int config_get_global_contrast( config_t *ct );
 int config_get_global_saturation( config_t *ct );
 int config_get_global_hue( config_t *ct );
-const char *config_get_audio_mode( config_t *ct );
 const char *config_get_xmltv_file( config_t *ct );
 const char *config_get_xmltv_language( config_t *ct );
 double config_get_overscan( config_t *ct );
+const char *config_get_matte( config_t *ct );
 int config_get_check_freq_present( config_t *ct );
 int config_get_usexds( config_t *ct );
 int config_get_invert( config_t *ct );
@@ -195,6 +195,7 @@ int config_get_show_taglines( config_t *ct );
 int config_get_square_pixels( config_t *ct );
 const char *config_get_alsa_inputdev( config_t *ct );
 const char *config_get_alsa_outputdev( config_t *ct );
+int config_get_alsa_latency( config_t *ct );
 
 #ifdef __cplusplus
 };
