@@ -615,7 +615,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
             }
 
             vbi->current_chan = (b1 & 8) >> 3;
-            if( !bottom == vbi->wanttop ) {
+	    if( (!bottom) == vbi->wanttop ) {
                 if( vbi->chan != vbi->current_chan )
                     return 0;
             } else return 0;
@@ -636,7 +636,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                      vbi->current_ital, vbi->current_ul, vbi->current_colour,
                      vbi->current_indent, vbi->current_row );
 
-            if( !bottom == vbi->wanttop && 
+	    if( (!bottom) == vbi->wanttop &&
                 vbi->current_chan == vbi->chan && 
                 vbi->current_istext == vbi->wanttext ) {
 
@@ -681,7 +681,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
 
             if( vbi->verbose ) fprintf( stderr, "Tab Offset: %d columns\n", b2 & 3 );
             if( vbi->wanttext && vbi->current_istext && 
-                vbi->current_chan == vbi->chan && !bottom == vbi->wanttop ) {
+		vbi->current_chan == vbi->chan && (!bottom) == vbi->wanttop ) {
                 vbiscreen_tab( vbi->vs, b2 & 3 );
             }
             vbi->lastcode = ( b1 << 8) | b2;
@@ -705,7 +705,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
             switch( code ) {
             case 0: /* POP-UP */
                 if( !vbi->wanttext && vbi->current_chan == vbi->chan &&
-                    !bottom == vbi->wanttop ) {
+		    (!bottom) == vbi->wanttop ) {
                     if( vbi->verbose )
                         fprintf( stderr, "Pop-Up\n");
                     vbi->indent = vbi->current_indent;
@@ -718,7 +718,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 break;
             case 5: /* ROLL UP 2 */ 
                 if( !vbi->wanttext && vbi->current_chan == vbi->chan &&
-                    !bottom == vbi->wanttop ) {
+		    (!bottom) == vbi->wanttop ) {
                     if( vbi->verbose )
                         fprintf( stderr, "Roll-Up 2 (RU2)\n");
                     vbi->indent = vbi->current_indent;
@@ -731,7 +731,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 break;
             case 6: /* ROLL UP 3 */ 
                 if( !vbi->wanttext && vbi->current_chan == vbi->chan &&
-                    !bottom == vbi->wanttop ) {
+		    (!bottom) == vbi->wanttop ) {
                     if( vbi->verbose )
                         fprintf( stderr, "Roll-Up 3 (RU3)\n");
                     vbi->indent = vbi->current_indent;
@@ -744,7 +744,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 break;
             case 7: /* ROLL UP 4 */ 
                 if( !vbi->wanttext && vbi->current_chan == vbi->chan &&
-                    !bottom == vbi->wanttop ) {
+		    (!bottom) == vbi->wanttop ) {
                     if( vbi->verbose )
                         fprintf( stderr, "Roll-Up 4 (RU4)\n");
                     vbi->indent = vbi->current_indent;
@@ -757,7 +757,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 break;
             case 9: /* PAINT-ON */
                 if( !vbi->wanttext && vbi->current_chan == vbi->chan &&
-                    !bottom == vbi->wanttop ) {
+		    (!bottom) == vbi->wanttop ) {
                     if( vbi->verbose )
                         fprintf( stderr, "Paint-On\n");
                     vbi->indent = vbi->current_indent;
@@ -770,7 +770,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 break;
             case 10:/* TEXT */
                 if( vbi->wanttext && vbi->current_chan == vbi->chan &&
-                    !bottom == vbi->wanttop ) {
+		    (!bottom) == vbi->wanttop ) {
                     if( vbi->verbose )
                         fprintf( stderr, "Text Restart\n");
                     vbi->indent = vbi->current_indent;
@@ -783,7 +783,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 break;
             case 11:/* TEXT */
                 if( vbi->wanttext && vbi->current_chan == vbi->chan &&
-                    !bottom == vbi->wanttop ) {
+		    (!bottom) == vbi->wanttop ) {
                     if( vbi->verbose )
                         fprintf( stderr, "Resume Text Display\n");
                     vbi->indent = vbi->current_indent;
@@ -803,7 +803,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
             if( !bottom && vbi->lastcode == ( (b1 << 8) | b2 ) ) {
                 vbi->lastcount = (vbi->lastcount + 1) % 2;
             }
-            if( !bottom == vbi->wanttop && vbi->current_chan == vbi->chan &&
+	    if( (!bottom) == vbi->wanttop && vbi->current_chan == vbi->chan &&
                 vbi->current_istext == vbi->wanttext ) {
                 if( vbi->verbose )
                     fprintf( stderr, "Backspace\n");
@@ -820,7 +820,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
             if( !bottom && vbi->lastcode == ( (b1 << 8) | b2 ) ) {
                 vbi->lastcount = (vbi->lastcount + 1) % 2;
             }
-            if( !bottom == vbi->wanttop && vbi->current_chan == vbi->chan &&
+	    if( (!bottom) == vbi->wanttop && vbi->current_chan == vbi->chan &&
                 vbi->current_istext == vbi->wanttext ) {
                 if( vbi->verbose )
                     fprintf( stderr, "Delete to End of Row\n");
@@ -845,7 +845,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
             switch( code ) {
             case 12:
                 /* Show buffer 1, Fill buffer 2 */
-                if( !bottom == vbi->wanttop && 
+		if( (!bottom) == vbi->wanttop &&
                     vbi->current_chan == vbi->chan && 
                     vbi->current_istext == vbi->wanttext ) {
                     if( vbi->verbose )
@@ -854,7 +854,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 }
                 break;
             case 13:
-                if( !bottom == vbi->wanttop && 
+		if( (!bottom) == vbi->wanttop &&
                     vbi->current_chan == vbi->chan && 
                     vbi->current_istext == vbi->wanttext ) {
                     if( vbi->verbose )
@@ -863,7 +863,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 }
                 break;
             case 14:
-                if( !bottom == vbi->wanttop && 
+		if( (!bottom) == vbi->wanttop &&
                     vbi->current_chan == vbi->chan && 
                     vbi->current_istext == vbi->wanttext ) {
                     if( vbi->verbose )
@@ -873,7 +873,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
                 break;
             case 15:
                 /* Show buffer 2, Fill Buffer 1 */
-                if( !bottom == vbi->wanttop && 
+		if( (!bottom) == vbi->wanttop &&
                     vbi->current_chan == vbi->chan && 
                     vbi->current_istext == vbi->wanttext ) {
                     if( vbi->verbose )
@@ -913,7 +913,7 @@ int ProcessLine( vbidata_t *vbi, uint8_t *s, int bottom )
         return 0;
     }
 
-    if( !bottom != vbi->wanttop || vbi->current_chan != vbi->chan || 
+    if( (!bottom) != vbi->wanttop || vbi->current_chan != vbi->chan ||
         vbi->current_istext != vbi->wanttext ) {
         return 0;
     }
