@@ -461,8 +461,11 @@ void vbiscreen_erase_non_displayed( vbiscreen_t *vs )
     if( vs->verbose ) fprintf( stderr, "vbiscreen: Erase non-displayed.\n");
 
     if( vs->captions && vs->style == POP_UP ) {
-        memset( vs->buffers + vs->curbuffer * COLS * ROWS + vs->cury * COLS, 0, COLS );
-/*        clear_hidden_pop( vs );*/
+        /* When a multi-row caption has been popped up, as in the scene where
+	   Bilbo gives Sting to Frodo in the Fellowship of the Rings, it's
+	   insufficient to clear just the current row. */
+/*        memset( vs->buffers + vs->curbuffer * COLS * ROWS + vs->cury * COLS, 0, COLS ); */
+        clear_hidden_pop( vs );
     } else if( vs->captions && vs->style && vs->style <= ROLL_4 ) {
         clear_hidden_roll( vs );
     }
