@@ -19,6 +19,7 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -38,6 +39,7 @@
 #include <linux/videodev2.h>
 #include "videoinput.h"
 #include "mixer.h"
+#include "get_media_devices.h"
 
 /**
  * How long to wait when we lose a signal, or acquire a signal.
@@ -123,6 +125,12 @@ int videoinput_get_time_per_field( int norm )
     }
 }
 
+static int videoinput_get_unmute_delay( int norm )
+{
+    /* The msp3400 needs some time before it provides stable audio */
+    return 400000 / videoinput_get_time_per_field( norm );
+}
+
 typedef struct capture_buffer_s
 {
     struct v4l2_buffer vidbuf;
@@ -135,6 +143,7 @@ struct videoinput_s
 {
     int verbose;
     int grab_fd;
+    char v4ldev[ PATH_MAX ];
     char drivername[ 64 ];
     char shortdriver[ 64 ];
 
@@ -158,6 +167,7 @@ struct videoinput_s
     int hasaudio;
     int audiomode;
     int change_muted;
+    int change_muted_delay;
     int user_muted;
     int hw_muted;
 
@@ -312,14 +322,68 @@ int videoinput_buffer_invalid( videoinput_t *vidin, int frameid )
     return vidin->capbuffers[ frameid ].free;
 }
 
-videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
-                              int volume, int norm, int verbose, char *error_string )
+int videoinput_open_v4l_device( videoinput_t *vidin, const char *path,
+				int caps, const char *type)
+{
+    struct v4l2_capability caps_v4l2;
+    int fd;
+
+    /* First, open the device. */
+    fd = open( path, O_RDWR );
+    if( fd == -1 ) {
+	fprintf( stderr, "videoinput: Cannot open device %s: %s\n",
+		 path, strerror( errno ) );
+    }
+
+    /**
+     * Next, ask for its capabilities.  This will also confirm it's a V4L2
+     * device.
+     */
+    if( ioctl( fd, VIDIOC_QUERYCAP, &caps_v4l2 ) < 0 ) {
+	/* Can't get V4L2 capabilities, maybe this is a V4L1 device? */
+	fprintf( stderr, "videoinput: %s: V4L1 devices not supported\n", path );
+	close( fd );
+	return -1;
+    }
+
+    if( (caps_v4l2.capabilities & caps) != caps ) {
+	if( vidin->verbose ) {
+	    fprintf( stderr, "videoinput: %s: does not have %s caps\n",
+		     path, type );
+	}
+	close( fd );
+	return -1;
+    }
+
+    if( vidin->verbose ) {
+	fprintf( stderr, "videoinput: Using video4linux2 driver '%s', card '%s' (bus %s).\n"
+			 "videoinput: Version is %u, capabilities %x.\n",
+		 caps_v4l2.driver, caps_v4l2.card, caps_v4l2.bus_info,
+		 caps_v4l2.version, caps_v4l2.capabilities );
+    }
+    vidin->isv4l2 = 1;
+    snprintf( vidin->v4ldev, sizeof( vidin->v4ldev ), "%s", path );
+    snprintf( vidin->drivername, sizeof( vidin->drivername ), "%s [%s/%s/%u]",
+	      caps_v4l2.driver, caps_v4l2.card,
+	      caps_v4l2.bus_info, caps_v4l2.version );
+    snprintf( vidin->shortdriver, sizeof( vidin->shortdriver ), "%s",
+	      caps_v4l2.driver );
+
+    return fd;
+}
+
+videoinput_t *videoinput_new( config_t *cfg, int norm, int verbose,
+			      char *error_string )
 {
     videoinput_t *vidin = malloc( sizeof( videoinput_t ) );
-    struct v4l2_capability caps_v4l2;
     struct v4l2_input in;
     int i;
 
+    const char *v4l_device = config_get_v4l_device( cfg );
+    int capwidth = config_get_inputwidth( cfg );
+    int volume = config_get_audio_boost( cfg );
+    int user_muted = config_get_muted( cfg );
+
     if( capwidth & 1 ) {
         capwidth -= 1;
         if( verbose ) {
@@ -355,7 +419,8 @@ videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
     vidin->signal_recover_wait = 0;
     vidin->signal_acquire_wait = 0;
     vidin->change_muted = 1;
-    vidin->user_muted = 0;
+    vidin->change_muted_delay = videoinput_get_unmute_delay( norm );
+    vidin->user_muted = user_muted;
     vidin->hw_muted = 1;
     vidin->hasaudio = 1;
     vidin->audiomode = 0;
@@ -366,44 +431,65 @@ videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
     memset( vidin->drivername, 0, sizeof( vidin->drivername ) );
     memset( vidin->shortdriver, 0, sizeof( vidin->shortdriver ) );
 
-    /* First, open the device. */
-    vidin->grab_fd = open( v4l_device, O_RDWR );
-    if( vidin->grab_fd < 0 ) {
-        fprintf( stderr, "videoinput: Cannot open capture device %s: %s\n",
-                 v4l_device, strerror( errno ) );
-        sprintf( error_string, "%s", strerror( errno ) );
-        free( vidin );
-        return 0;
-    }
+    vidin->grab_fd = -1;
 
-    /**
-     * Next, ask for its capabilities.  This will also confirm it's a V4L2 
-     * device. 
-     */
-    if( ioctl( vidin->grab_fd, VIDIOC_QUERYCAP, &caps_v4l2 ) < 0 ) {
-        /* Can't get V4L2 capabilities, maybe this is a V4L1 device? */
-	fprintf(stderr, "V4L1 devices not supported\n");
-	sprintf( error_string, "V4L1 devices not supported" );
-	close( vidin->grab_fd );
+#ifdef __linux__ /* Because this depends on get_media_devices.c */
+    if( !strcmp( v4l_device, "auto") ) {
+	char path[PATH_MAX];
+	const char *type;
+	int caps;
+	void *md;
+
+	md = discover_media_devices();
+	if( !md ) {
+	    sprintf( error_string, "Cannot enumerate video devices" );
+	    free( vidin );
+	    return 0;
+        }
+
+	/* Step 1: try TV cards first */
+	type = "analog TV";
+	caps = V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_TUNER;
+retry:
+	v4l_device = NULL;
+	while( 1 ) {
+	    v4l_device = get_associated_device( md, v4l_device, MEDIA_V4L_VIDEO,
+						NULL, NONE );
+	    if( !v4l_device )
+		break; /* No more video devices to try */
+	    snprintf( path, PATH_MAX, "/dev/%s", v4l_device );
+	    if( verbose )
+		fprintf(stderr, "videoinput-auto: trying: %s... \n", path);
+	    vidin->grab_fd = videoinput_open_v4l_device( vidin, path, caps,
+							 type );
+	    if( vidin->grab_fd != -1 ) {
+		fprintf( stderr, "videoinput-auto: using %s device %s\n",
+			 type, path);
+		break;
+	    }
+	}
+
+	/* Step 2: try grabber devices and webcams */
+	if( vidin->grab_fd == -1 &&
+		caps == (V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_TUNER) ) {
+	    type = "capture";
+	    caps = V4L2_CAP_VIDEO_CAPTURE;
+	    goto retry;
+	}
+
+	free_media_devices(md);
+    } else
+#endif
+	vidin->grab_fd = videoinput_open_v4l_device( vidin, v4l_device,
+						     V4L2_CAP_VIDEO_CAPTURE,
+						     "capture" );
+
+    if( vidin->grab_fd == -1 ) {
+	sprintf( error_string, "Cannot open/use video device %s", v4l_device );
 	free( vidin );
 	return 0;
-    } else {
-        if( vidin->verbose ) {
-            fprintf( stderr, "videoinput: Using video4linux2 driver '%s', card '%s' (bus %s).\n"
-                             "videoinput: Version is %u, capabilities %x.\n",
-                     caps_v4l2.driver, caps_v4l2.card, caps_v4l2.bus_info,
-                     caps_v4l2.version, caps_v4l2.capabilities );
-        }
-        vidin->isv4l2 = 1;
-        snprintf( vidin->drivername, sizeof( vidin->drivername ),
-                  "%s [%s/%s/%u]",
-                  caps_v4l2.driver, caps_v4l2.card,
-                  caps_v4l2.bus_info, caps_v4l2.version );
-        snprintf( vidin->shortdriver, sizeof( vidin->shortdriver ),
-                  "%s", caps_v4l2.driver );
     }
 
-
     vidin->numinputs = 0;
     in.index = vidin->numinputs;
 
@@ -698,8 +784,13 @@ void videoinput_set_saturation_relative( videoinput_t *vidin, int offset )
     }
 }
 
-static void videoinput_do_mute( videoinput_t *vidin, int mute )
+static void videoinput_do_mute( videoinput_t *vidin )
 {
+    int mute = vidin->user_muted || vidin->change_muted;
+
+    if( mute )
+	mixer_mute( 1 );
+
     if( vidin->hasaudio && mute != vidin->hw_muted ) {
 	struct v4l2_control control;
 
@@ -717,6 +808,9 @@ static void videoinput_do_mute( videoinput_t *vidin, int mute )
 	}
         vidin->hw_muted = mute;
     }
+
+    if( !mute )
+	mixer_mute( 0 );
 }
 
 /* freqKHz is in KHz (duh) */
@@ -736,14 +830,14 @@ void videoinput_set_tuner_freq( videoinput_t *vidin, int freqKHz )
 
         if( !vidin->tunerlow ) {
             frequency /= 1000; /* switch to MHz */
-        }
+	}
 
-        vidin->change_muted = 1;
-        mixer->mute( 1 );
-        videoinput_do_mute( vidin, vidin->user_muted || vidin->change_muted );
-        vidin->cur_tuner_state = TUNER_STATE_SIGNAL_DETECTED;
-        vidin->signal_acquire_wait = SIGNAL_ACQUIRE_DELAY;
-        vidin->signal_recover_wait = 0;
+	vidin->change_muted = 1;
+	vidin->change_muted_delay = videoinput_get_unmute_delay( vidin->norm );
+	videoinput_do_mute( vidin );
+	vidin->cur_tuner_state = TUNER_STATE_SIGNAL_DETECTED;
+	vidin->signal_acquire_wait = SIGNAL_ACQUIRE_DELAY;
+	vidin->signal_recover_wait = 0;
 
 
 	wasstreaming = vidin->is_streaming;
@@ -906,28 +1000,27 @@ int videoinput_check_for_signal( videoinput_t *vidin, int check_freq_present )
                 vidin->signal_acquire_wait--;
                 break;
             } else {
-                vidin->cur_tuner_state = TUNER_STATE_HAS_SIGNAL;
-            }
-        default:
-            if( vidin->change_muted ) {
-                vidin->change_muted = 0;
-                videoinput_do_mute( vidin, vidin->user_muted || vidin->change_muted );
-                mixer->mute( 0 );
-            }
-            break;
-        }
+		vidin->cur_tuner_state = TUNER_STATE_HAS_SIGNAL;
+	    }
+	default:
+	    if( vidin->change_muted && --vidin->change_muted_delay == 0 ) {
+		vidin->change_muted = 0;
+		videoinput_do_mute( vidin );
+	    }
+	    break;
+	}
     } else {
         switch( vidin->cur_tuner_state ) {
         case TUNER_STATE_HAS_SIGNAL:
         case TUNER_STATE_SIGNAL_DETECTED:
-            vidin->cur_tuner_state = TUNER_STATE_SIGNAL_LOST;
-            vidin->signal_recover_wait = SIGNAL_RECOVER_DELAY;
-            vidin->change_muted = 1;
-            mixer->mute( 1 );
-            videoinput_do_mute( vidin, vidin->user_muted || vidin->change_muted );
-        case TUNER_STATE_SIGNAL_LOST:
-            if( vidin->signal_recover_wait ) {
-                vidin->signal_recover_wait--;
+	    vidin->cur_tuner_state = TUNER_STATE_SIGNAL_LOST;
+	    vidin->signal_recover_wait = SIGNAL_RECOVER_DELAY;
+	    vidin->change_muted = 1;
+	    vidin->change_muted_delay = videoinput_get_unmute_delay( vidin->norm );
+	    videoinput_do_mute( vidin );
+	case TUNER_STATE_SIGNAL_LOST:
+	    if( vidin->signal_recover_wait ) {
+		vidin->signal_recover_wait--;
                 break;
             } else {
                 vidin->cur_tuner_state = TUNER_STATE_NO_SIGNAL;
@@ -960,7 +1053,7 @@ void videoinput_switch_pal_secam( videoinput_t *vidin, int norm )
 void videoinput_mute( videoinput_t *vidin, int mute )
 {
     vidin->user_muted = mute;
-    videoinput_do_mute( vidin, vidin->user_muted || vidin->change_muted );
+    videoinput_do_mute( vidin );
 }
 
 int videoinput_get_muted( videoinput_t *vidin )
@@ -1037,6 +1130,11 @@ const char *videoinput_get_input_name( videoinput_t *vidin )
     return vidin->inputname;
 }
 
+const char *videoinput_get_v4ldev( videoinput_t *vidin )
+{
+    return vidin->v4ldev;
+}
+
 const char *videoinput_get_driver_name( videoinput_t *vidin )
 {
     return vidin->drivername;
@@ -1050,4 +1148,3 @@ void videoinput_set_capture_volume( videoinput_t *vidin, int volume )
 				     ((double) vidin->volume) / 100.0 );
     }
 }
-
