@@ -24,6 +24,7 @@
 #else
 # include <stdint.h>
 #endif
+#include "tvtimeconf.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -108,8 +109,7 @@ const char *videoinput_get_audio_mode_name( videoinput_t *vidin, int mode );
  * The verbose flag indicates we should print to stderr when things go
  * bad, and maybe for some helpful information messages.
  */
-videoinput_t *videoinput_new( const char *v4l_device, int capwidth,
-                              int volume, int norm, int verbose,
+videoinput_t *videoinput_new( config_t *cfg, int norm, int verbose,
                               char *error_string );
 
 /**
@@ -277,6 +277,11 @@ void videoinput_set_pal_audio_mode( videoinput_t *vidin, int amode );
  */
 int videoinput_get_pal_audio_mode( videoinput_t *vidin );
 
+/**
+ * Returns the path to the v4ldev.
+ */
+const char *videoinput_get_v4ldev( videoinput_t *vidin );
+
 /**
  * Returns the name of the capture card driver.
  */
