@@ -43,10 +43,11 @@
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
+#include <X11/XKBlib.h>
 #include <X11/cursorfont.h>
 #include <X11/extensions/XShm.h>
-#ifdef HAVE_XTESTEXTENSION
-#include <X11/extensions/XTest.h>
+#ifdef HAVE_XSSEXTENSION
+#include <X11/extensions/scrnsaver.h>
 #endif
 
 #include "xfullscreen.h"
@@ -66,8 +67,9 @@ static int screen;
 static Window wm_window;
 static Window fs_window;
 static Window output_window;
+static XSizeHints hint;
 static GC gc;
-static int have_xtest;
+static int have_xss;
 static int output_width, output_height;
 static int output_aspect;
 static int output_on_root;
@@ -85,7 +87,6 @@ static int matte_width;
 static int matte_height;
 static int alwaysontop;
 static int has_focus;
-static int wm_is_metacity;
 static int use_square_pixels;
 static xfullscreen_t *xf;
 
@@ -107,10 +108,6 @@ static Atom wm_delete_window;
 static Atom xawtv_station;
 static Atom xawtv_remote;
 
-#ifdef HAVE_XTESTEXTENSION
-static KeyCode kc_shift_l; /* Fake key to send. */
-#endif
-
 static area_t video_area;
 static area_t window_area;
 static area_t scale_area;
@@ -211,7 +208,7 @@ static const char *tvtime_icon =
 "                                "
 "                                " ;
 
-static void load_icon( uint32_t *data )
+static void load_icon( long *data )
 {
     int i;
 
@@ -248,12 +245,12 @@ static void x11_wait_mapped( Display *dpy, Window win )
     } while ( (event.type != MapNotify) || (event.xmap.event != win) );
 }
 
-static int have_xtestextention( void )
+static int have_xssextention( void )
 {  
-#ifdef HAVE_XTESTEXTENSION
-    int dummy1, dummy2, dummy3, dummy4;
+#ifdef HAVE_XSSEXTENSION
+    int dummy1, dummy2;
   
-    return (XTestQueryExtension( display, &dummy1, &dummy2, &dummy3, &dummy4 ) == True);
+    return (XScreenSaverQueryExtension( display, &dummy1, &dummy2 ) == True);
 #endif
     return 0;
 }
@@ -343,14 +340,26 @@ static int xv_get_height_for_width( int window_width )
     return rounded_int_division( window_width * n, d );
 }
 
-static void x11_northwest_gravity( Display *dpy, Window win )
+static void x11_aspect_hint( Display *dpy, Window win, int aspect_width, int aspect_height )
 {
     XSizeHints hints;
-    hints.flags = PWinGravity;
-    hints.win_gravity = NorthWestGravity;
+
+    hints.flags = PAspect;
+    hints.min_aspect.x = aspect_width;
+    hints.min_aspect.y = aspect_height;
+    hints.max_aspect.x = aspect_width;
+    hints.max_aspect.y = aspect_height;
+
     XSetWMNormalHints( dpy, win, &hints );
 }
 
+static void x11_northwest_gravity( Display *dpy, Window win )
+{
+    hint.flags |= PWinGravity;
+    hint.win_gravity = NorthWestGravity;
+    XSetWMNormalHints( dpy, win, &hint );
+}
+
 static void x11_grab_fullscreen_input( Display *dpy, Window win )
 {
     int tries = 10;
@@ -825,13 +834,12 @@ static void calculate_video_area( void )
 }
 
 
-int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
+int xcommon_open_display( const char *user_geometry, int aspect, int border, int verbose )
 {
     Pixmap curs_pix;
     XEvent xev;
-    XSizeHints hint;
     XClassHint classhint;
-    XColor curs_col;
+    XColor curs_col = { 0 };
     XSetWindowAttributes xswa;
     const char *hello = "tvtime";
     unsigned long mask;
@@ -843,7 +851,7 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
     output_aspect = aspect;
     output_height = 576;
 
-    have_xtest = 0;
+    have_xss = 0;
     output_on_root = 0;
     has_ewmh_state_fullscreen = 0;
     has_ewmh_state_above = 0;
@@ -858,7 +866,6 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
     matte_height = 0;
     alwaysontop = 0;
     has_focus = 0;
-    wm_is_metacity = 0;
 
     display = XOpenDisplay( 0 );
     if( !display ) {
@@ -927,14 +934,17 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
         xfullscreen_print_summary( xf );
     }
 
-#ifdef HAVE_XTESTEXTENSION
-    kc_shift_l = XKeysymToKeycode( display, XK_Shift_L );
-#endif
-    have_xtest = have_xtestextention();
-    if( have_xtest && xcommon_verbose ) {
-        fprintf( stderr, "xcommon: Have XTest, will use it to ping the screensaver.\n" );
+    have_xss = have_xssextention();
+    if( have_xss && xcommon_verbose ) {
+        fprintf( stderr, "xcommon: Have XSS, will use it to disable the screensaver.\n" );
     }
 
+#ifdef HAVE_XSSEXTENSION
+    if ( have_xss ) {
+        XScreenSaverSuspend( display, True );
+    }
+#endif
+
     /* Initially, get the best width for our height. */
     output_width = xv_get_width_for_height( output_height );
 
@@ -1005,7 +1015,7 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
 
     /* Set the icon on the window. */
     {
-        uint32_t *data = malloc( 1026 * 4 );
+        long *data = malloc( (32*32 + 2) * sizeof(long) );
         if( data ) {
             load_icon( data );
             XChangeProperty( display, wm_window, net_wm_icon, cardinal, 32,
@@ -1023,13 +1033,6 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
             fprintf( stderr, "xcommon: Window manager is %s and is EWMH compliant.\n", wmname );
         }
 
-        if( !strcasecmp( wmname, "metacity" ) ) {
-            if( xcommon_verbose ) {
-                fprintf( stderr, "xcommon: You are using metacity.  Disabling aspect ratio hints\n"
-                                 "xcommon: since most deployed versions of metacity are still broken.\n" );
-            }
-            wm_is_metacity = 1;
-        }
         free( wmname );
 
 
@@ -1061,6 +1064,23 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
 
     XSetStandardProperties( display, wm_window, hello, hello, None, 0, 0, &hint );
 
+    Atom prop = None;
+    MWMHints mwmhints;
+    if (!border) {
+        prop = XInternAtom( display, "_MOTIF_WM_INFO", True );
+        if( prop == None ) {
+            mwmhints.flags = 0;
+        } else {
+            mwmhints.flags = MWM_HINTS_DECORATIONS;
+            mwmhints.decorations = 0;
+	}
+    }
+
+    if( mwmhints.flags ) {
+        prop = XInternAtom( display, "_MOTIF_WM_HINTS", False );
+        XChangeProperty( display, wm_window, prop, prop, 32, PropModeReplace, (unsigned char *) &mwmhints, PROP_MWM_HINTS_ELEMENTS );
+    }
+
     /* The class hint is useful for window managers like WindowMaker. */
     classhint.res_class = "tvtime";
     classhint.res_name = "TVWindow";
@@ -1072,6 +1092,7 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
     XSetWMProtocols( display, wm_window, &wm_delete_window, 1 );
 
     calculate_video_area();
+    x11_aspect_hint( display, wm_window, video_area.width, video_area.height );
 
     XMapWindow( display, output_window );
     XMapWindow( display, wm_window );
@@ -1083,13 +1104,7 @@ int xcommon_open_display( const char *user_geometry, int aspect, int verbose )
     XMaskEvent( display, StructureNotifyMask, &xev );
 
     /* Create a 1 pixel cursor to use in full screen mode */
-    curs_pix = XCreatePixmap( display, output_window, 1, 1, 1 );
-    curs_col.pixel = 0;
-    curs_col.red = 0;
-    curs_col.green = 0;
-    curs_col.blue = 0;
-    curs_col.flags = 0;
-    curs_col.pad = 0;
+    curs_pix = XCreateBitmapFromData( display, output_window, (const char []){0}, 1, 1 );
     nocursor = XCreatePixmapCursor( display, curs_pix, curs_pix, &curs_col, &curs_col, 1, 1 );
     XDefineCursor( display, output_window, nocursor );
     XSetIconName( display, wm_window, "tvtime" );
@@ -1112,15 +1127,7 @@ void xcommon_ping_screensaver( void )
     gettimeofday( &curtime, 0 );
     if( timediff( &curtime, &last_ping_time ) > SCREENSAVER_PING_TIME ) { 
         last_ping_time = curtime;
-#ifdef HAVE_XTESTEXTENSION
-        if( have_xtest ) {
-            XTestFakeKeyEvent( display, kc_shift_l, True, CurrentTime );
-            XTestFakeKeyEvent( display, kc_shift_l, False, CurrentTime );
-        } else 
-#endif
-        {
-            XResetScreenSaver( display );
-        }
+        XResetScreenSaver( display );
     }
 }
 
@@ -1366,6 +1373,7 @@ int xcommon_toggle_aspect( void )
 {
     output_aspect = !output_aspect;
     calculate_video_area();
+    x11_aspect_hint( display, wm_window, video_area.width, video_area.height );
     xcommon_clear_screen();
     return output_aspect;
 }
@@ -1401,7 +1409,6 @@ void xcommon_poll_events( input_t *in )
     int reconfigure = 0;
     int reconfwidth = 0;
     int reconfheight = 0;
-    int getfocus = 0;
     int motion = 0;
     int motion_x = 0;
     int motion_y = 0;
@@ -1472,9 +1479,6 @@ void xcommon_poll_events( input_t *in )
             XUndefineCursor( display, output_window );
             motion_timeout = 30;
             break;
-        case EnterNotify:
-            getfocus = 1;
-            break;
         case FocusIn:
             has_focus = 1;
             break;
@@ -1577,9 +1581,9 @@ void xcommon_poll_events( input_t *in )
             /* if( event.xkey.state & ShiftMask ) arg |= I_SHIFT; */
             /* this alternate approach allows handling of keys like '<' and '>' -- mrallen */
             if( event.xkey.state & ShiftMask ) {
-                mykey = XKeycodeToKeysym( display, event.xkey.keycode, 1 );
+                mykey = XkbKeycodeToKeysym( display, event.xkey.keycode, 0, 1 );
             } else {
-                mykey = XKeycodeToKeysym( display, event.xkey.keycode, 0 );
+                mykey = XkbKeycodeToKeysym( display, event.xkey.keycode, 0, 0 );
             }
             if( event.xkey.state & ControlMask ) arg |= I_CTRL;
             if( event.xkey.state & Mod1Mask ) arg |= I_META;
@@ -1658,7 +1662,6 @@ void xcommon_poll_events( input_t *in )
                 input_callback( in, I_BUTTONPRESS, event.xbutton.button );
                 lastpresstime = event.xbutton.time;
             }
-            getfocus = 1;
             break;
         case ButtonRelease:
             xcommon_update_server_time( event.xbutton.time );
@@ -1666,7 +1669,6 @@ void xcommon_poll_events( input_t *in )
                 input_callback( in, I_BUTTONRELEASE, event.xbutton.button );
                 lastreleasetime = event.xbutton.time;
             }
-            getfocus = 1;
             break;
         default: break;
         }
@@ -1676,10 +1678,6 @@ void xcommon_poll_events( input_t *in )
         input_callback( in, I_MOUSEMOVE, ((motion_x & 0xffff) << 16 | (motion_y & 0xffff)) );
     }
 
-    if( getfocus ) {
-        XSetInputFocus( display, wm_window, RevertToPointerRoot, CurrentTime );
-    }
-
     if( !has_ewmh_state_fullscreen && output_fullscreen ) {
         /**
          * We used to check if we still have focus too.
@@ -1715,6 +1713,11 @@ void xcommon_poll_events( input_t *in )
 
 void xcommon_close_display( void )
 {
+#ifdef HAVE_XSSEXTENSION
+    if ( have_xss ) {
+        XScreenSaverSuspend( display, False );
+    }
+#endif
     XDestroyWindow( display, output_window );
     XDestroyWindow( display, wm_window );
     XDestroyWindow( display, fs_window );
@@ -1810,6 +1813,11 @@ void xcommon_set_fullscreen_position( int pos )
 
 void xcommon_set_matte( int width, int height )
 {
+    if( width ) {
+        x11_aspect_hint( display, wm_window, width, height );
+    } else {
+        x11_aspect_hint( display, wm_window, video_area.width, video_area.height );
+    }
     matte_width = width;
     matte_height = height;
     calculate_video_area();
